#Copyright (c) 2021 Eito Tan

#This software is released under the MIT License.
#http://opensource.org/licenses/mit-license.php

import bpy

bl_info = {
    "name": "Converting shader settings",
    "author": "Eito Tan",
    "version": (1, 1),
    "blender": (3, 0, 0),
    "location": "3Dビューポート > オブジェクト > 適用",
    "description": "Set shader for imported 3D models",
    "warning": "",
    "support": "TESTING",
    "doc_url": "",
    "tracker_url": "",
    "category": "Object"
}


class port_shader_settings(bpy.types.Operator):

    bl_idname = "shader.port_shader_settings"
    bl_label = "Port shader settings"
    bl_description = "Port shader settings"
    bl_options = {'REGISTER', 'UNDO'}


    shader_node_name: bpy.props.StringProperty(
        name="Shader Node",
        description="Set Shader Node Name",
        default='simple_skin_shader'
    )
    image_node_name: bpy.props.StringProperty(
        name="Image Node",
        description="Set Image Node Name",
        default='mmd_base_tex'
    )


    # メニューを実行したときに呼ばれる関数
    def execute(self, context):
        
        #set operator properties
        
        #error check
        try:
            tmp_str = self.shader_node_name
            test_node_tree = bpy.data.node_groups[tmp_str]
        except Exception as e:
            print(e)
            self.report({'INFO'}, "Shader Node not found.")
            return {'FINISHED'}

        for s in context.active_object.material_slots:

            if "SKIN" not in s.name:
                node_tree = s.material.node_tree
                    
                #nodes.remove(nodes["プリンシプルBSDF"]  )

                #delete previous created node
                for n in node_tree.nodes:
                    if n.type == "OUTPUT_MATERIAL":
                        node_tree.nodes.remove(n)
                    elif n.label =="PortShaderSettings":
                        node_tree.nodes.remove(n)
                            
                #add shader if not skin
                nnode = node_tree.nodes.new('ShaderNodeBsdfPrincipled')
                #Mark for deletion for prevent duplication
                nnode.label = "PortShaderSettings"
                
                #add output node
                outnode_all = node_tree.nodes.new('ShaderNodeOutputMaterial')

                #delete link
                #node_tree.links.remove()

                #link texture node  to skin shader
                tmp_str = self.image_node_name
                try:
                    node_tree.links.new(node_tree.nodes[tmp_str].outputs[0], nnode.inputs[0])
                    node_tree.links.new(node_tree.nodes[tmp_str].outputs[1], nnode.inputs[21])
                    node_tree.links.new(nnode.outputs[0], outnode_all.inputs[0])
                except Exception as e:
                    print(e)
                    self.report({'INFO'}, "Image Node not found.")
                    return {'FINISHED'}

                nnode.location = (200, 800)
                outnode_all.location = (400, 800)
                
            else:
                node_tree = s.material.node_tree
                #for n in node_tree.nodes:
                #    print(n.name)
                    
                #nodes.remove(nodes["プリンシプルBSDF"]  )

                #delete previous created node group
                for n in node_tree.nodes:
                    try:
                        tmp_str = self.shader_node_name
                        if n.type == "OUTPUT_MATERIAL":
                            node_tree.nodes.remove(n)
                        elif n.node_tree == bpy.data.node_groups[tmp_str]:
                            node_tree.nodes.remove(n)
                            
                    except AttributeError as e:
                        print(e)

                #add shader for skin
                skinnode = node_tree.nodes.new('ShaderNodeGroup')
                tmp_str = self.shader_node_name
                skinnode.node_tree = bpy.data.node_groups[tmp_str]
                skinnode.inputs[1].default_value = (0.6, 0.6, 0.6, 1)
                skinnode.inputs[2].default_value = (0.479, 0.053, 0.065, 1)
                skinnode.inputs[3].default_value = 0.784
                
                #add output node
                outnode_cycles = node_tree.nodes.new('ShaderNodeOutputMaterial')
                #outnode_cycles.target = "CYCLES"
                #outnode_eevee = node_tree.nodes.new('ShaderNodeOutputMaterial')
                #outnode_eevee.target = "EEVEE"
                #delete link
                #node_tree.links.remove()

                #link texture node to skin shader
                tmp_str = self.image_node_name
                node_tree.links.new(node_tree.nodes[tmp_str].outputs[0], skinnode.inputs[0])
                node_tree.links.new(skinnode.outputs[0], outnode_cycles.inputs[0])
                #node_tree.links.new(skinnode.outputs[1], outnode_cycles.inputs[2])
                
                #node_tree.links.new(skinnode.outputs[2], outnode_eevee.inputs[0])

                skinnode.location = (200, 800)
                outnode_cycles.location = (400, 800)
                #outnode_eevee.location = (400, 700)

        return {'FINISHED'}

# Set Menu
def menu_fn(self, context):
    self.layout.separator()
    self.layout.operator(port_shader_settings.bl_idname)

# Blenderに登録するクラス
classes = [
    port_shader_settings,
]

# アドオン有効化時の処理
def register():
    for c in classes:
        bpy.utils.register_class(c)
    bpy.types.VIEW3D_MT_object_apply.append(menu_fn)
    print("Addon-port_shader_settings Enabled")


# アドオン無効化時の処理
def unregister():
    bpy.types.VIEW3D_MT_object_apply.remove(menu_fn)
    for c in classes:
        bpy.utils.unregister_class(c)
    print("Addon-port_shader_settings Disabled")


# メイン処理
if __name__ == "__main__":
    register()